@@ -44,11 +44,7 @@ namespace {
  */
 bool isTesting()
 {
-#if QT_VERSION < 0x050100
-    return !qgetenv("COPYQ_TEST_ID").isEmpty();
-#else
     return !qEnvironmentVariableIsEmpty("COPYQ_TEST_ID");
-#endif
 }
 
 /**
@@ -137,6 +133,12 @@ App::App(
     , m_started(false)
     , m_closed(false)
 {
+    QObject::connect(m_app, &QCoreApplication::aboutToQuit, [this]() { exit(); });
+
+#ifdef Q_OS_UNIX
+    startUnixSignalHandler();
+#endif
+
     QString session("copyq");
     if ( !sessionName.isEmpty() ) {
         session += "-" + sessionName;
@@ -155,11 +157,6 @@ App::App(
 #ifdef HAS_TESTS
     initTests();
 #endif
-
-#ifdef Q_OS_UNIX
-    if ( !UnixSignalHandler::create(m_app) )
-        log( QString("Failed to create handler for Unix signals!"), LogError );
-#endif
 }
 
 App::~App()
@@ -171,7 +168,7 @@ App::~App()
 
 void App::restoreSettings(bool canModifySettings)
 {
-    m_app->setProperty("CopyQ_server", canModifySettings);
+    Settings::canModifySettings = canModifySettings;
 
     createPlatformNativeInterface()->loadSettings();
 
@@ -201,9 +198,9 @@ void App::exit(int exitCode)
     if (!m_started)
         ::exit(exitCode);
 
-    QCoreApplication::exit(exitCode);
     m_exitCode = exitCode;
     m_closed = true;
+    QCoreApplication::exit(exitCode);
 }
 
 bool App::wasClosed() const
