@@ -29,7 +29,6 @@
 #include <QBuffer>
 #include <QClipboard>
 #include <QDropEvent>
-#include <QDir>
 #include <QImage>
 #include <QImageWriter>
 #include <QKeyEvent>
@@ -38,7 +37,6 @@
 #include <QObject>
 #include <QPoint>
 #include <QProcess>
-#include <QTemporaryFile>
 #include <QTextCodec>
 #include <QThread>
 #include <QTimer>
@@ -46,7 +44,7 @@
 #include <QWidget>
 
 // This is needed on X11 when retrieving lots of data from clipboard.
-#if QT_VERSION >= 0x050000 && defined(COPYQ_WS_X11)
+#ifdef COPYQ_WS_X11
 #   define PROCESS_EVENTS_BEFORE_CLIPBOARD_DATA
 #endif
 
@@ -174,6 +172,12 @@ bool hasNonInternalFormats(const QStringList &formats) {
     return false;
 }
 
+QByteArray thisClipboardOwnerData()
+{
+    return qgetenv("COPYQ_SESSION_NAME")
+            + " " + QByteArray::number(QCoreApplication::applicationPid());
+}
+
 } // namespace
 
 bool isMainThread()
@@ -292,7 +296,7 @@ QMimeData* createMimeData(const QVariantMap &data)
     // Don't set clipboard owner if monitor is only used to set clipboard for tests.
     if ( !qApp->property("CopyQ_testing").toBool() )
 #endif
-        newClipboardData->setData( mimeOwner, qgetenv("COPYQ_SESSION_NAME") );
+        newClipboardData->setData( mimeOwner, thisClipboardOwnerData() );
 
     // Set image data.
     const QStringList formats =
@@ -429,23 +433,12 @@ void renameToUnique(QString *name, const QStringList &names)
         *name = baseName + " (" + QString::number(++i) + ')';
 }
 
-bool openTemporaryFile(QTemporaryFile *file, const QString &suffix)
-{
-    const QString tmpFileName = "CopyQ.XXXXXX" + suffix;
-    const QString tmpPath = QDir( QDir::tempPath() ).absoluteFilePath(tmpFileName);
-
-    file->setFileTemplate(tmpPath);
-    file->setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner);
-
-    return file->open();
-}
-
 void initSingleShotTimer(QTimer *timer, int milliseconds, const QObject *object, const char *slot)
 {
     timer->setSingleShot(true);
     timer->setInterval(milliseconds);
     if (object && slot)
-        QObject::connect( timer, SIGNAL(timeout()), object, slot );
+        QObject::connect( timer, SIGNAL(timeout()), object, slot, Qt::UniqueConnection );
 }
 
 QString dataToText(const QByteArray &bytes, const QString &mime)
@@ -564,3 +557,14 @@ void acceptDrag(QDropEvent *event)
         event->acceptProposedAction();
     }
 }
+
+bool ownsClipboardData(ClipboardMode mode)
+{
+    return clipboardOwnerData(mode) == thisClipboardOwnerData();
+}
+
+QByteArray clipboardOwnerData(ClipboardMode mode)
+{
+    const auto data = clipboardData(mode);
+    return data ? data->data(mimeOwner) : QByteArray();
+}
