@@ -36,18 +36,9 @@ class QPoint;
 class QProcess;
 class QString;
 class QStringList;
-class QTemporaryFile;
 class QTimer;
 class QWidget;
 
-#if QT_VERSION < 0x050000
-#   ifdef Q_WS_WIN
-#       define Q_OS_WIN
-#   elif defined Q_WS_MAC
-#       define Q_OS_MAC
-#   endif
-#endif
-
 #if !defined(COPYQ_WS_X11) && !defined(Q_OS_WIN) && !defined(Q_OS_MAC)
 #   define NO_GLOBAL_SHORTCUTS
 #endif
@@ -61,6 +52,10 @@ bool isMainThread();
 
 const QMimeData *clipboardData(ClipboardMode mode = ClipboardMode::Clipboard);
 
+bool ownsClipboardData(ClipboardMode mode);
+
+QByteArray clipboardOwnerData(ClipboardMode mode);
+
 /** Clone data for given formats (text or HTML will be UTF8 encoded). */
 QVariantMap cloneData(const QMimeData &data, QStringList formats);
 
@@ -109,8 +104,6 @@ QString textLabelForData(const QVariantMap &data);
 
 void renameToUnique(QString *name, const QStringList &names);
 
-bool openTemporaryFile(QTemporaryFile *file, const QString &suffix);
-
 void initSingleShotTimer(QTimer *timer, int milliseconds, const QObject *object = nullptr, const char *slot = nullptr);
 
 QString dataToText(const QByteArray &bytes, const QString &mime);
