@@ -23,6 +23,7 @@
 #include "common/contenttype.h"
 #include "common/log.h"
 #include "common/mimetypes.h"
+#include "common/temporaryfile.h"
 #include "common/textdata.h"
 #include "gui/clipboarddialog.h"
 #include "gui/iconfactory.h"
@@ -277,6 +278,10 @@ ClipboardBrowser::ClipboardBrowser(
     initSingleShotTimer( &m_timerUpdateItemWidgets, 0, this, SLOT(updateItemWidgets()) );
     initSingleShotTimer( &m_timerUpdateCurrent, 0, this, SLOT(updateCurrent()) );
 
+    m_timerDragDropScroll.setInterval(20);
+    connect( &m_timerDragDropScroll, &QTimer::timeout,
+             this, &ClipboardBrowser::dragDropScroll );
+
     // ScrollPerItem doesn't work well with hidden items
     setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 
@@ -293,7 +298,6 @@ ClipboardBrowser::ClipboardBrowser(
 ClipboardBrowser::~ClipboardBrowser()
 {
     delete m_editor.data();
-    d.invalidateCache();
     saveUnsavedItems();
 }
 
@@ -580,6 +584,7 @@ void ClipboardBrowser::processDragAndDropEvent(QDropEvent *event)
 {
     acceptDrag(event);
     m_dragTargetRow = getDropRow(event->pos());
+    dragDropScroll();
 }
 
 int ClipboardBrowser::dropIndexes(const QModelIndexList &indexes)
@@ -632,13 +637,8 @@ void ClipboardBrowser::moveToTop(const QModelIndex &index)
         return;
 
     const auto data = index.data(contentType::data).toMap();
-    if ( m_itemSaver->canMoveItems(QList<QModelIndex>() << index) ) {
-#if QT_VERSION < 0x050000
-        m.moveRow(index.row(), 0);
-#else
+    if ( m_itemSaver->canMoveItems(QList<QModelIndex>() << index) )
         m.moveRow(QModelIndex(), index.row(), QModelIndex(), 0);
-#endif
-    }
 }
 
 void ClipboardBrowser::maybeEmitEditingFinished()
@@ -662,6 +662,34 @@ QModelIndex ClipboardBrowser::firstUnpinnedIndex() const
     return QModelIndex();
 }
 
+void ClipboardBrowser::dragDropScroll()
+{
+    if (m_dragTargetRow == -1) {
+        m_timerDragDropScroll.stop();
+        return;
+    }
+
+    if (!m_timerDragDropScroll.isActive()) {
+        m_timerDragDropScroll.start();
+        return;
+    }
+
+    const auto y = mapFromGlobal(QCursor::pos()).y();
+    const auto h = viewport()->contentsRect().height();
+    const auto d = h / 4;
+    const auto scrollAmount =
+            (h < y + d) ? (y + d - h) / 4
+          : (y < d) ? -(d - y) / 4
+          : 0;
+
+    if (scrollAmount != 0) {
+        QScrollBar *v = verticalScrollBar();
+        v->setValue(v->value() + scrollAmount);
+    } else {
+        m_timerDragDropScroll.stop();
+    }
+}
+
 QVariantMap ClipboardBrowser::copyIndex(const QModelIndex &index) const
 {
     auto data = index.data(contentType::data).toMap();
@@ -787,11 +815,7 @@ QPixmap ClipboardBrowser::renderItemPreview(const QModelIndexList &indexes, int
     if (h == 0)
         return QPixmap();
 
-#if QT_VERSION >= 0x050000
     const auto ratio = devicePixelRatio();
-#else
-    const auto ratio = 1;
-#endif
     const auto frameLineWidth = 2 * ratio;
 
     const auto height = qMin(maxHeight, h + s + 2 * frameLineWidth);
@@ -799,9 +823,7 @@ QPixmap ClipboardBrowser::renderItemPreview(const QModelIndexList &indexes, int
     const QSize size(width, height);
 
     QPixmap pix(size * ratio);
-#if QT_VERSION >= 0x050000
     pix.setDevicePixelRatio(ratio);
-#endif
 
     QPainter p(&pix);
 
@@ -1294,9 +1316,10 @@ void ClipboardBrowser::filterItems(const QRegExp &re)
     d.setSearch(re);
 
     // If search string is a number, highlight item in that row.
-    bool ok;
-    m_filterRow = re.pattern().toInt(&ok);
-    if (!ok)
+    bool filterByRowNumber = !m_sharedData->numberSearch;
+    if (filterByRowNumber)
+        m_filterRow = re.pattern().toInt(&filterByRowNumber);
+    if (!filterByRowNumber)
         m_filterRow = -1;
 
     int row = 0;
@@ -1307,7 +1330,7 @@ void ClipboardBrowser::filterItems(const QRegExp &re)
     for ( ; row < length(); ++row )
         hideFiltered(row);
 
-    if ( ok && m_filterRow >= 0 && m_filterRow < m.rowCount() )
+    if ( filterByRowNumber && m_filterRow >= 0 && m_filterRow < m.rowCount() )
         setCurrentIndex( index(m_filterRow) );
 }
 
