@@ -76,49 +76,31 @@ enum class ImportOptions {
 };
 
 struct MainWindowOptions {
-    MainWindowOptions()
-        : confirmExit(true)
-        , viMode(false)
-        , trayCommands(false)
-        , trayCurrentTab(false)
-        , trayTabName()
-        , trayItems(5)
-        , trayImages(true)
-        , trayMenuOpenOnLeftClick(false)
-        , transparency(0)
-        , transparencyFocused(0)
-        , hideTabs(false)
-        , hideMainWindow(false)
-        , itemActivationCommands(ActivateCloses)
-        , clearFirstTab(false)
-        , trayItemPaste(true)
-        , clipboardTab()
-    {}
-
     bool activateCloses() const { return itemActivationCommands & ActivateCloses; }
     bool activateFocuses() const { return itemActivationCommands & ActivateFocuses; }
     bool activatePastes() const { return itemActivationCommands & ActivatePastes; }
 
-    bool confirmExit;
-    bool viMode;
-    bool trayCommands;
-    bool trayCurrentTab;
+    bool confirmExit = true;
+    bool viMode = false;
+    bool trayCommands = false;
+    bool trayCurrentTab = false;
     QString trayTabName;
-    int trayItems;
-    bool trayImages;
-    bool trayMenuOpenOnLeftClick;
-    int transparency;
-    int transparencyFocused;
+    int trayItems = 5;
+    bool trayImages = true;
+    bool trayMenuOpenOnLeftClick = false;
+    int transparency = 0;
+    int transparencyFocused = 0;
 
-    bool hideTabs;
+    bool hideTabs = false;
 
-    bool hideMainWindow;
+    bool hideMainWindow = false;
+    bool closeOnUnfocus = false;
 
-    int itemActivationCommands;
+    int itemActivationCommands = ActivateCloses;
 
-    bool clearFirstTab;
+    bool clearFirstTab = false;
 
-    bool trayItemPaste;
+    bool trayItemPaste = true;
 
     QString clipboardTab;
 };
@@ -375,11 +357,9 @@ public slots:
     void runInternalAction(Action *action);
     bool isInternalActionId(int id) const;
 
-    /** Set clipboard. */
     void setClipboard(const QVariantMap &data, ClipboardMode mode);
-
-    /** Set clipboard and synchronize selection if needed. */
     void setClipboard(const QVariantMap &data);
+    void setClipboardAndWait(const QVariantMap &data, ClipboardMode mode);
 
     /** Show/hide main window. Return true only if window is shown. */
     bool toggleVisible();
@@ -449,11 +429,6 @@ public slots:
      * @return Last key clicks sequence number received by widgets.
      */
     uint lastReceivedKeyClicks();
-
-    /**
-     * Remove all tab and reset configuration (work only in test session).
-     */
-    void resetTestSession(const QString &clipboardTabName);
 #endif
 
 signals:
@@ -482,17 +457,7 @@ protected:
 
     bool focusNextPrevChild(bool next) override;
 
-#if QT_VERSION < 0x050000
-#   ifdef COPYQ_WS_X11
-    bool x11Event(XEvent *event) override;
-#   elif defined(Q_OS_WIN)
-    bool winEvent(MSG *message, long *result) override;
-#   elif defined(Q_OS_MAC)
-    bool macEvent(EventHandlerCallRef caller, EventRef event) override;
-#   endif
-#else
     bool nativeEvent(const QByteArray &eventType, void *message, long *result) override;
-#endif
 
 private slots:
     ClipboardBrowser *getTabForMenu();
@@ -500,8 +465,8 @@ private slots:
     void addMenuItems(const QString &searchText);
     void addTrayMenuItems(const QString &searchText);
     void trayActivated(QSystemTrayIcon::ActivationReason reason);
-    void onMenuActionTriggered(uint itemHash, bool omitPaste);
-    void onTrayActionTriggered(uint itemHash, bool omitPaste);
+    void onMenuActionTriggered(const QVariantMap &data, bool omitPaste);
+    void onTrayActionTriggered(const QVariantMap &data, bool omitPaste);
     void findNextOrPrevious();
     void tabChanged(int current, int previous);
     void saveTabPositions();
@@ -513,6 +478,8 @@ private slots:
     void onFilterChanged(const QRegExp &re);
     void createTrayIfSupported();
 
+    void raiseLastWindowAfterMenuClosed();
+
     /** Update WId for paste and last focused window if needed. */
     void updateFocusWindows();
 
@@ -604,6 +571,8 @@ private:
 
     void updateTrayMenu();
 
+    void updateTrayMenuClipboard();
+
     void updateNotifications();
 
     void updateWindowTransparency(bool mouseOver = false);
@@ -671,7 +640,7 @@ private:
     QAction *actionForMenuItem(int id, QWidget *parent, Qt::ShortcutContext context);
 
     void addMenuItems(TrayMenu *menu, ClipboardBrowser *c, int maxItemCount, const QString &searchText);
-    void onMenuActionTriggered(ClipboardBrowser *c, uint itemHash, bool omitPaste);
+    void activateMenuItem(ClipboardBrowser *c, const QVariantMap &data, bool omitPaste);
     QWidget *toggleMenu(TrayMenu *menu, QPoint pos);
     QWidget *toggleMenu(TrayMenu *menu);
 
@@ -688,6 +657,7 @@ private:
 
     QMenu *m_menuItem;
     TrayMenu *m_trayMenu;
+    QPointer<QAction> m_trayMenuClipboardAction;
 
     QSystemTrayIcon *m_tray;
 
@@ -713,6 +683,7 @@ private:
     QTimer m_timerTrayIconSnip;
     QTimer m_timerSaveTabPositions;
     QTimer m_timerHideWindowIfNotActive;
+    QTimer m_timerRaiseLastWindowAfterMenuClosed;
 
     NotificationDaemon *m_notifications;
 
