@@ -22,10 +22,9 @@
 #include "common/contenttype.h"
 #include "common/common.h"
 #include "common/display.h"
+#include "common/textdata.h"
 #include "gui/icons.h"
 #include "gui/iconfactory.h"
-#include "platform/platformnativeinterface.h"
-#include "platform/platformwindow.h"
 
 #include <QApplication>
 #include <QKeyEvent>
@@ -34,6 +33,8 @@
 
 namespace {
 
+const QIcon iconClipboard() { return getIcon("clipboard", IconPaste); }
+
 bool canActivate(const QAction &action)
 {
     return !action.isSeparator() && action.isEnabled();
@@ -71,6 +72,7 @@ TrayMenu::TrayMenu(QWidget *parent)
     , m_clipboardItemActionCount(0)
     , m_omitPaste(false)
     , m_viMode(false)
+    , m_numberSearch(false)
 {
     initSingleShotTimer( &m_timerUpdateActiveAction, 0, this, SLOT(updateActiveAction()) );
 }
@@ -86,7 +88,7 @@ void TrayMenu::addClipboardItemAction(const QModelIndex &index, bool showImages,
     const QVariantMap data = index.data(contentType::data).toMap();
     QAction *act = addAction(QString());
 
-    act->setData(index.data(contentType::hash));
+    act->setData(index.data(contentType::data));
 
     insertAction(m_clipboardItemActionsSeparator, act);
 
@@ -168,6 +170,11 @@ void TrayMenu::setViModeEnabled(bool enabled)
     m_viMode = enabled;
 }
 
+void TrayMenu::setNumberSearchEnabled(bool enabled)
+{
+    m_numberSearch = enabled;
+}
+
 void TrayMenu::keyPressEvent(QKeyEvent *event)
 {
     const int key = event->key();
@@ -211,7 +218,7 @@ void TrayMenu::keyPressEvent(QKeyEvent *event)
                 const QString txt = event->text();
                 if ( !txt.isEmpty() && txt[0].isPrint() ) {
                     // Activate item at row when number is entered.
-                    if (m_searchText.isEmpty()) {
+                    if ( !m_numberSearch && m_searchText.isEmpty() ) {
                         bool ok;
                         const int row = txt.toInt(&ok);
                         if (ok && row < m_clipboardItemActionCount) {
@@ -292,6 +299,22 @@ void TrayMenu::search(const QString &text)
     emit searchRequest(m_viMode ? m_searchText.mid(1) : m_searchText);
 }
 
+void TrayMenu::markItemInClipboard(const QVariantMap &clipboardData)
+{
+    const auto text = getTextData(clipboardData);
+
+    for ( auto action : actions() ) {
+        const auto actionData = action->data().toMap();
+        const auto itemText = getTextData(actionData);
+        if ( !itemText.isEmpty() ) {
+            const auto hideIcon = itemText != text;
+            const auto isIconHidden = action->icon().isNull();
+            if ( isIconHidden != hideIcon )
+                action->setIcon(hideIcon ? QIcon() : iconClipboard());
+        }
+    }
+}
+
 void TrayMenu::setSearchMenuItem(const QString &text)
 {
     if ( m_searchAction.isNull() ) {
@@ -311,16 +334,16 @@ void TrayMenu::onClipboardItemActionTriggered()
     QAction *act = qobject_cast<QAction *>(sender());
     Q_ASSERT(act != nullptr);
 
-    QVariant actionData = act->data();
-    Q_ASSERT( actionData.isValid() );
-
-    uint hash = actionData.toUInt();
-    emit clipboardItemActionTriggered(hash, m_omitPaste);
+    const auto actionData = act->data().toMap();
+    emit clipboardItemActionTriggered(actionData, m_omitPaste);
     close();
 }
 
 void TrayMenu::updateActiveAction()
 {
+    if ( isVisible() && activeAction() != nullptr )
+        return;
+
     const auto action = firstEnabledAction(this);
     if (action != nullptr)
         setActiveAction(action);
