@@ -49,18 +49,7 @@ using ItemSaverPtr = std::shared_ptr<ItemSaverInterface>;
 class ItemScriptableFactoryInterface;
 using ItemScriptableFactoryPtr = std::shared_ptr<ItemScriptableFactoryInterface>;
 
-#define COPYQ_PLUGIN_ITEM_LOADER_ID "org.CopyQ.ItemPlugin.ItemLoader/1.0"
-
-#if QT_VERSION < 0x050000
-#   define Q_PLUGIN_METADATA(x)
-#else
-#   if defined(Q_EXPORT_PLUGIN)
-#       undef Q_EXPORT_PLUGIN
-#       undef Q_EXPORT_PLUGIN2
-#   endif
-#   define Q_EXPORT_PLUGIN(x)
-#   define Q_EXPORT_PLUGIN2(x,y)
-#endif
+#define COPYQ_PLUGIN_ITEM_LOADER_ID "com.github.hluk.copyq.itemloader/3.4.0"
 
 /**
  * Handles item in list.
