@@ -21,11 +21,9 @@
 
 #include "app/applicationexceptionhandler.h"
 #include "common/log.h"
-#include "copyqpasteboardmime.h"
 #include "foregroundbackgroundfilter.h"
 #include "macplatformwindow.h"
 #include "platform/mac/macactivity.h"
-#include "urlpasteboardmime.h"
 #include "macclipboard.h"
 
 #include <QApplication>
@@ -39,23 +37,6 @@
 #include <Carbon/Carbon.h>
 
 namespace {
-    class ClipboardApplication : public QApplication
-    {
-    public:
-        ClipboardApplication(int &argc, char **argv)
-            : QApplication(argc, argv)
-            , m_pasteboardMime()
-            , m_pasteboardMimeUrl(QLatin1String("public.url"))
-            , m_pasteboardMimeFileUrl(QLatin1String("public.file-url"))
-        {
-        }
-
-    private:
-        CopyQPasteboardMime m_pasteboardMime;
-        UrlPasteboardMime m_pasteboardMimeUrl;
-        UrlPasteboardMime m_pasteboardMimeFileUrl;
-    };
-
     template<typename T> inline T* objc_cast(id from)
     {
         if ([from isKindOfClass:[T class]]) {
@@ -144,6 +125,21 @@ namespace {
     {
         return QCoreApplication::applicationDirPath() + "/../Resources/" + path;
     }
+
+    template <typename QtApplication>
+    class Activity
+        : public MacActivity
+        , public ApplicationExceptionHandler<QtApplication>
+    {
+    public:
+        Activity(int &argc, char **argv, ActivityType type, const QString &reason)
+            : MacActivity(type, reason)
+            , ApplicationExceptionHandler<QtApplication>(argc, argv)
+        {
+            [NSApp setActivationPolicy:NSApplicationActivationPolicyProhibited];
+        }
+    };
+
 } // namespace
 
 PlatformPtr createPlatformNativeInterface()
@@ -162,8 +158,7 @@ QCoreApplication *MacPlatform::createConsoleApplication(int &argc, char **argv)
 
 QApplication *MacPlatform::createServerApplication(int &argc, char **argv)
 {
-    MacActivity activity(MacActivity::Background, "CopyQ Server");
-    QApplication *app = new ApplicationExceptionHandler<ClipboardApplication>(argc, argv);
+    QApplication *app = new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ Server");
 
     // Switch the app to foreground when in foreground
     ForegroundBackgroundFilter::installFilter(app);
@@ -173,14 +168,17 @@ QApplication *MacPlatform::createServerApplication(int &argc, char **argv)
 
 QApplication *MacPlatform::createMonitorApplication(int &argc, char **argv)
 {
-    MacActivity activity(MacActivity::Background, "CopyQ clipboard monitor");
-    return new ApplicationExceptionHandler<ClipboardApplication>(argc, argv);
+    return new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard monitor");
+}
+
+QApplication *MacPlatform::createClipboardProviderApplication(int &argc, char **argv)
+{
+    return new Activity<QApplication>(argc, argv, MacActivity::Background, "CopyQ clipboard provider");
 }
 
 QCoreApplication *MacPlatform::createClientApplication(int &argc, char **argv)
 {
-    MacActivity activity(MacActivity::User, "CopyQ Client");
-    return new ApplicationExceptionHandler<QCoreApplication>(argc, argv);
+    return new Activity<QCoreApplication>(argc, argv, MacActivity::User, "CopyQ Client");
 }
 
 PlatformClipboardPtr MacPlatform::clipboard()
