@@ -109,7 +109,7 @@ void migrateConfigToAppDir()
         QSettings::setPath(QSettings::IniFormat, QSettings::UserScope, dir.absolutePath());
         Settings newSettings;
 
-        if ( Settings::canModifySettings() && newSettings.isEmpty() ) {
+        if ( Settings::canModifySettings && newSettings.isEmpty() ) {
             COPYQ_LOG("Migrating configuration to application directory.");
             const QString newConfigPath = QDir::cleanPath(newSettings.fileName() + "/..");
 
@@ -291,6 +291,11 @@ QApplication *WinPlatform::createMonitorApplication(int &argc, char **argv)
     return createGuiApplication(argc, argv);
 }
 
+QApplication *WinPlatform::createClipboardProviderApplication(int &argc, char **argv)
+{
+    return createGuiApplication(argc, argv);
+}
+
 QCoreApplication *WinPlatform::createClientApplication(int &argc, char **argv)
 {
     return createApplication<QCoreApplication>(argc, argv);
