@@ -20,6 +20,7 @@
 #include "x11platform.h"
 
 #include "app/applicationexceptionhandler.h"
+#include "common/log.h"
 #include "common/textdata.h"
 
 #include <QApplication>
@@ -29,10 +30,10 @@
 #include <QStringList>
 #include <QVariant>
 #include <QWidget>
+#include <QX11Info>
 
 #include "x11platformwindow.h"
 #include "x11platformclipboard.h"
-#include "x11displayguard.h"
 
 #include <X11/Xatom.h>
 
@@ -42,6 +43,18 @@ namespace {
 
 int (*old_xio_errhandler)(Display *) = nullptr;
 
+const char *defaultDesktopFileContent =
+R"([Desktop Entry]
+Name=CopyQ
+Icon=copyq
+GenericName=Clipboard Manager
+Type=Application
+Terminal=false
+X-KDE-autostart-after=panel
+X-KDE-StartupNotify=false
+X-KDE-UniqueApplet=true
+)";
+
 // Try to handle X11 fatal error gracefully.
 int copyq_xio_errhandler(Display *display)
 {
@@ -73,6 +86,20 @@ QString getDesktopFilename()
 }
 #endif
 
+void printFileError(const QFile &file, const char *message, LogLevel logLevel = LogError)
+{
+    log( QString("%1 \"%2\": %3")
+         .arg(message)
+         .arg(file.fileName())
+         .arg(file.errorString()), logLevel );
+}
+
+void maybePrintFileError(const QFile &file, const char *message)
+{
+    if (file.error() != QFile::NoError)
+        printFileError(file, message);
+}
+
 } // namespace
 
 PlatformPtr createPlatformNativeInterface()
@@ -84,21 +111,19 @@ X11Platform::~X11Platform() = default;
 
 PlatformWindowPtr X11Platform::getWindow(WId winId)
 {
-    auto d = std::make_shared<X11DisplayGuard>();
-    if (!d->display())
+    if (!QX11Info::isPlatformX11())
         return PlatformWindowPtr();
 
-    std::unique_ptr<X11PlatformWindow> window(new X11PlatformWindow(d, winId));
+    std::unique_ptr<X11PlatformWindow> window(new X11PlatformWindow(winId));
     return PlatformWindowPtr(window->isValid() ? window.release() : nullptr);
 }
 
 PlatformWindowPtr X11Platform::getCurrentWindow()
 {
-    auto d = std::make_shared<X11DisplayGuard>();
-    if (!d->display())
+    if (!QX11Info::isPlatformX11())
         return PlatformWindowPtr();
 
-    std::unique_ptr<X11PlatformWindow> window(new X11PlatformWindow(d));
+    std::unique_ptr<X11PlatformWindow> window(new X11PlatformWindow());
     return PlatformWindowPtr(window->isValid() ? window.release() : nullptr);
 }
 
@@ -148,38 +173,45 @@ void X11Platform::setAutostartEnabled(bool enable)
 
     const QString filename = getDesktopFilename();
 
-    QFile desktopFile(filename);
+    const auto autostartPath = QDir::cleanPath(filename + "/..");
+    QDir autostartDir(autostartPath);
+    if ( !autostartDir.mkpath(".") ) {
+        log( QString("Failed to create autostart path \"%1\"").arg(autostartPath) );
+        return;
+    }
 
+    QFile desktopFile2(filename + ".new");
+    if ( !desktopFile2.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text) ) {
+        printFileError(desktopFile2, "Failed to create new desktop file");
+        return;
+    }
+
+    QRegExp re("^(Hidden|X-GNOME-Autostart-enabled|Exec)\\s*=\\s*");
+
+    QFile desktopFile(filename);
     bool createUserDesktopFile = !desktopFile.exists();
     if (createUserDesktopFile)
         desktopFile.setFileName(COPYQ_DESKTOP_FILE);
 
-    if ( !desktopFile.open(QIODevice::ReadOnly | QIODevice::Text) )
-        return;
-
-    QFile desktopFile2(filename + ".new");
-    if ( !QDir(QDir::cleanPath(filename + "/..")).mkpath(".")
-         || !desktopFile2.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text) )
-    {
-        return;
-    }
-
-    QRegExp re("^(Hidden|X-GNOME-Autostart-enabled)\\s*=\\s*");
-
-    while ( !desktopFile.atEnd() ) {
-        QString line = getTextData(desktopFile.readLine());
-        QString cmd = "\"" + QApplication::applicationFilePath() + "\"";
-        if ( line.startsWith("Exec=") ) {
-            const QString sessionName = qApp->property("CopyQ_session_name").toString();
-            if ( !sessionName.isEmpty() )
-                cmd.append(" -s " + sessionName);
-            desktopFile2.write("Exec=" + cmd.toUtf8() + "\n");
-        } else if ( re.indexIn(line) == -1 ) {
-            desktopFile2.write(line.toUtf8());
+    if ( desktopFile.open(QIODevice::ReadOnly | QIODevice::Text) ) {
+        while ( !desktopFile.atEnd() ) {
+            QString line = getTextData(desktopFile.readLine());
+            if ( re.indexIn(line) == -1 )
+                desktopFile2.write(line.toUtf8());
         }
+        desktopFile.close();
+        maybePrintFileError(desktopFile, "Failed to read desktop file");
+    } else {
+        // Installed desktop file not found (can happen when running tests).
+        printFileError(desktopFile, "Failed to open desktop file", LogNote);
+        desktopFile2.write(defaultDesktopFileContent);
     }
 
-    desktopFile.close();
+    QString cmd = "\"" + QApplication::applicationFilePath() + "\"";
+    const QString sessionName = qApp->property("CopyQ_session_name").toString();
+    if ( !sessionName.isEmpty() )
+        cmd.append(" -s " + sessionName);
+    desktopFile2.write("Exec=" + cmd.toUtf8() + "\n");
 
     desktopFile2.write("Hidden=");
     desktopFile2.write(enable ? "False" : "True");
@@ -191,6 +223,8 @@ void X11Platform::setAutostartEnabled(bool enable)
 
     QFile::remove(filename);
     desktopFile2.rename(filename);
+
+    maybePrintFileError(desktopFile2, "Failed to write desktop file");
 #else
     Q_UNUSED(enable);
 #endif
@@ -203,7 +237,8 @@ QCoreApplication *X11Platform::createConsoleApplication(int &argc, char **argv)
 
 QApplication *X11Platform::createServerApplication(int &argc, char **argv)
 {
-    old_xio_errhandler = XSetIOErrorHandler(copyq_xio_errhandler);
+    if (QX11Info::isPlatformX11())
+        old_xio_errhandler = XSetIOErrorHandler(copyq_xio_errhandler);
     return new ApplicationExceptionHandler<QApplication>(argc, argv);
 }
 
@@ -212,6 +247,11 @@ QApplication *X11Platform::createMonitorApplication(int &argc, char **argv)
     return new ApplicationExceptionHandler<QApplication>(argc, argv);
 }
 
+QApplication *X11Platform::createClipboardProviderApplication(int &argc, char **argv)
+{
+    return new ApplicationExceptionHandler<QApplication>(argc, argv);
+}
+
 QCoreApplication *X11Platform::createClientApplication(int &argc, char **argv)
 {
     return new ApplicationExceptionHandler<QCoreApplication>(argc, argv);
@@ -219,8 +259,7 @@ QCoreApplication *X11Platform::createClientApplication(int &argc, char **argv)
 
 PlatformClipboardPtr X11Platform::clipboard()
 {
-    auto d = std::make_shared<X11DisplayGuard>();
-    return PlatformClipboardPtr(new X11PlatformClipboard(d));
+    return PlatformClipboardPtr(new X11PlatformClipboard());
 }
 
 QStringList X11Platform::getCommandLineArguments(int argc, char **argv)
