@@ -23,14 +23,12 @@
 
 #include "common/common.h"
 #include "common/mimetypes.h"
-#include "common/log.h"
-
-#include "x11displayguard.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
 #include <QClipboard>
+#include <QX11Info>
 
 namespace {
 
@@ -48,27 +46,12 @@ bool isSelectionIncomplete(Display *display)
     return event.xbutton.state & (Button1Mask | ShiftMask);
 }
 
-bool isClipboardEmpty(Display *display)
-{
-    static Atom atom = XInternAtom(display, "CLIPBOARD", False);
-    return XGetSelectionOwner(display, atom) == None;
-}
-
-bool isSelectionEmpty(Display *display)
-{
-    static Atom atom = XA_PRIMARY;
-    return XGetSelectionOwner(display, atom) == None;
-}
-
 } // namespace
 
-X11PlatformClipboard::X11PlatformClipboard(const std::shared_ptr<X11DisplayGuard> &d)
-    : d(d)
+X11PlatformClipboard::X11PlatformClipboard()
 {
     initSingleShotTimer( &m_timerCheckClipboard, 50, this, SLOT(onClipboardChanged()) );
     initSingleShotTimer( &m_timerCheckSelection, 100, this, SLOT(onSelectionChanged()) );
-    initSingleShotTimer( &m_timerResetClipboard, 500, this, SLOT(resetClipboard()) );
-    initSingleShotTimer( &m_timerResetSelection, 500, this, SLOT(resetSelection()) );
 }
 
 void X11PlatformClipboard::setFormats(const QStringList &formats)
@@ -97,26 +80,18 @@ void X11PlatformClipboard::onChanged(int mode)
 
 void X11PlatformClipboard::onClipboardChanged()
 {
-    m_timerResetClipboard.stop();
     const QVariantMap data = DummyClipboard::data(ClipboardMode::Clipboard, m_formats);
-    const bool foreignData = !ownsClipboardData(data);
-    if ( foreignData && maybeResetClipboard() )
-        return;
-
     if (m_clipboardData == data)
         return;
 
     m_clipboardData = data;
     emit changed(ClipboardMode::Clipboard);
 
-    // Check selection too if some signals where not delivered.
-    m_timerCheckSelection.start();
+    checkAgain();
 }
 
 void X11PlatformClipboard::onSelectionChanged()
 {
-    m_timerResetSelection.stop();
-
     // Qt clipboard data can be in invalid state after this call.
     if ( waitIfSelectionIncomplete() )
         return;
@@ -124,43 +99,23 @@ void X11PlatformClipboard::onSelectionChanged()
     // Always assume that only plain text can be in primary selection buffer.
     // Asking a app for bigger data when mouse selection changes can make the app hang for a moment.
     const QVariantMap data = DummyClipboard::data( ClipboardMode::Selection, QStringList(mimeText) );
-    const bool foreignData = !ownsClipboardData(data);
-    if ( foreignData && maybeResetSelection() )
-        return;
-
     if (m_selectionData == data)
         return;
 
     m_selectionData = data;
     emit changed(ClipboardMode::Selection);
 
-    // Check clipboard too if some signals where not delivered.
-    m_timerCheckClipboard.start();
-}
-
-void X11PlatformClipboard::resetClipboard()
-{
-    if (!m_clipboardData.isEmpty()) {
-        COPYQ_LOG("Resetting clipboard");
-        DummyClipboard::setData(ClipboardMode::Clipboard, m_clipboardData);
-    }
-}
-
-void X11PlatformClipboard::resetSelection()
-{
-    if (!m_selectionData.isEmpty()) {
-        COPYQ_LOG("Resetting selection");
-        DummyClipboard::setData(ClipboardMode::Selection, m_selectionData);
-    }
-
+    checkAgain();
 }
 
 bool X11PlatformClipboard::waitIfSelectionIncomplete()
 {
-    if (!d->display())
+    if (!QX11Info::isPlatformX11())
         return true;
 
-    if ( isSelectionIncomplete(d->display()) ) {
+    auto display = QX11Info::display();
+
+    if ( isSelectionIncomplete(display) ) {
         m_timerCheckSelection.start();
         return true;
     }
@@ -168,30 +123,10 @@ bool X11PlatformClipboard::waitIfSelectionIncomplete()
     return false;
 }
 
-bool X11PlatformClipboard::maybeResetClipboard()
+void X11PlatformClipboard::checkAgain()
 {
-    if (!d->display())
-        return false;
-
-    if ( m_clipboardData.isEmpty() || !isClipboardEmpty(d->display()) )
-        return false;
-
-    COPYQ_LOG("Clipboard is empty");
-    m_timerResetClipboard.start();
-
-    return true;
-}
-
-bool X11PlatformClipboard::maybeResetSelection()
-{
-    if (!d->display())
-        return false;
-
-    if ( m_selectionData.isEmpty() || !isSelectionEmpty(d->display()) )
-        return false;
-
-    COPYQ_LOG("Selection is empty");
-    m_timerResetSelection.start();
-
-    return true;
+    // Check clipboard and selection again if some signals where
+    // not delivered or older data was received after new one.
+    m_timerCheckClipboard.start();
+    m_timerCheckSelection.start();
 }
