@@ -39,7 +39,6 @@
 #include "scriptable/scriptableproxy.h"
 #include "scriptable/temporaryfileclass.h"
 #include "../qt/bytearrayclass.h"
-#include "../qxt/qxtglobal.h"
 
 #include <QApplication>
 #include <QClipboard>
@@ -479,23 +478,27 @@ bool matchData(const QRegExp &re, const QVariantMap &data, const QString &format
     return re.indexIn(text) != -1;
 }
 
+bool isInternalDataFormat(const QString &format)
+{
+    return format == mimeWindowTitle
+        || format == mimeItems
+        || format == mimeOwner
+        || format == mimeClipboardMode
+        || format == mimeCurrentTab
+        || format == mimeSelectedItems
+        || format == mimeCurrentItem
+        || format == mimeShortcut
+        || format == mimeOutputTab
+        || format == mimeSyncToClipboard
+        || format == mimeSyncToSelection;
+}
+
 QVariantMap copyWithoutInternalData(const QVariantMap &data) {
     QVariantMap newData;
     for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
-        if ( it.key() != mimeWindowTitle
-          && it.key() != mimeItems
-          && it.key() != mimeOwner
-          && it.key() != mimeClipboardMode
-          && it.key() != mimeCurrentTab
-          && it.key() != mimeSelectedItems
-          && it.key() != mimeCurrentItem
-          && it.key() != mimeShortcut
-          && it.key() != mimeOutputTab
-          && it.key() != mimeSyncToClipboard
-          && it.key() != mimeSyncToSelection)
-        {
-            newData.insert(it.key(), it.value());
-        }
+        const auto &format = it.key();
+        if ( !isInternalDataFormat(format) )
+            newData.insert(format, it.value());
     }
 
     return newData;
@@ -512,6 +515,61 @@ QStringList monitorFormatsToSave()
     return factory.formatsToSave();
 }
 
+#ifdef HAS_MOUSE_SELECTIONS
+class SynchronizeSelectionTimer : public QObject
+{
+public:
+    explicit SynchronizeSelectionTimer(ScriptableProxy *scriptableProxy)
+        : QObject()
+        , m_scriptableProxy(scriptableProxy)
+    {
+        m_timer.setSingleShot(true);
+        QObject::connect(
+            &m_timer, &QTimer::timeout,
+            this, &SynchronizeSelectionTimer::synchronize);
+    }
+
+    void start(ClipboardMode targetClipboardMode, const QVariantMap &data)
+    {
+        m_targetClipboardMode = targetClipboardMode;
+        m_data = data;
+        if ( !m_timer.isActive() )
+            m_timer.start(0);
+        syncLog("Started");
+    }
+
+    void reset()
+    {
+        m_data.clear();
+        m_timer.start(500);
+        syncLog("Reset");
+    }
+
+private:
+    void synchronize()
+    {
+        if ( m_data.isEmpty() )
+            return;
+
+        syncLog("Synchronizing");
+        m_scriptableProxy->setClipboard(m_data, m_targetClipboardMode);
+        syncLog("Synchronized");
+    }
+
+    void syncLog(const char *message) const
+    {
+        COPYQ_LOG( QString("Sync to %1: %2")
+                   .arg(m_targetClipboardMode == ClipboardMode::Clipboard ? "clipboard" : "selection")
+                   .arg(message) );
+    }
+
+    ScriptableProxy *m_scriptableProxy;
+    ClipboardMode m_targetClipboardMode = ClipboardMode::Clipboard;
+    QVariantMap m_data;
+    QTimer m_timer;
+};
+#endif // HAS_MOUSE_SELECTIONS
+
 } // namespace
 
 Scriptable::Scriptable(
@@ -678,7 +736,7 @@ void Scriptable::setCurrentPath(const QString &path)
         m_temporaryFileClass->setCurrentPath(path);
 }
 
-QString Scriptable::getFileName(const QString &fileName) const
+QString Scriptable::getAbsoluteFilePath(const QString &fileName) const
 {
     return QDir::isRelativePath(fileName) ? getCurrentPath() + '/' + fileName
                                           : fileName;
@@ -697,11 +755,26 @@ void Scriptable::throwError(const QString &errorMessage)
     currentContext->throwError( fromString(errorMessage + '\n') );
 }
 
+void Scriptable::throwSaveError(const QString &filePath)
+{
+    throwError( tr("Cannot save to file \"%1\"!").arg(filePath) );
+}
+
+void Scriptable::throwImportError(const QString &filePath)
+{
+    throwError( tr("Cannot import file \"%1\"!").arg(filePath) );
+}
+
 void Scriptable::sendMessageToClient(const QByteArray &message, int exitCode)
 {
     emit sendMessage(message, exitCode);
 }
 
+QScriptValue Scriptable::getGlobal()
+{
+    return m_engine->globalObject();
+}
+
 QScriptValue Scriptable::getPlugins()
 {
     // Load plugins on demand.
@@ -744,7 +817,6 @@ QScriptValue Scriptable::version()
     return tr(programName) + " " COPYQ_VERSION " (hluk@email.cz)\n"
             + tr("Built with: ")
             + "Qt " + QT_VERSION_STR +
-            + ", LibQxt " + QXT_VERSION_STR
             + '\n';
 }
 
@@ -1261,53 +1333,51 @@ void Scriptable::exportTab()
 {
     m_skipArguments = 1;
 
-    const QString &fileName = arg(0);
-    if ( fileName.isNull() ) {
+    const auto filePath = arg(0);
+    if ( filePath.isNull() )
         throwError(argumentError());
-    } else if ( !m_proxy->saveTab(getFileName(fileName)) ) {
-        throwError( tr("Cannot save to file \"%1\"!").arg(fileName) );
-    }
+    else if ( !m_proxy->saveTab(getAbsoluteFilePath(filePath)) )
+        throwSaveError(filePath);
 }
 
 void Scriptable::importTab()
 {
     m_skipArguments = 1;
 
-    const QString &fileName = arg(0);
-    if ( fileName.isNull() ) {
+    const auto filePath = arg(0);
+    if ( filePath.isNull() )
         throwError(argumentError());
-    } else if ( !m_proxy->loadTab(getFileName(fileName)) ) {
-        throwError(
-            tr("Cannot import file \"%1\"!").arg(fileName) );
-    }
+    else if ( !m_proxy->loadTab(getAbsoluteFilePath(filePath)) )
+        throwImportError(filePath);
 }
 
-QScriptValue Scriptable::importData()
+void Scriptable::importData()
 {
     m_skipArguments = 1;
 
-    const auto fileName = arg(0);
-    return m_proxy->importData(fileName);
+    const auto filePath = arg(0);
+    if ( filePath.isNull() )
+        throwError(argumentError());
+    else if ( !m_proxy->importData(getAbsoluteFilePath(filePath)) )
+        throwImportError(filePath);
 }
 
-QScriptValue Scriptable::exportData()
+void Scriptable::exportData()
 {
     m_skipArguments = 1;
 
-    const auto fileName = arg(0);
-    return m_proxy->exportData(fileName);
+    const auto filePath = arg(0);
+    if ( filePath.isNull() )
+        throwError(argumentError());
+    else if ( !m_proxy->exportData(getAbsoluteFilePath(filePath)) )
+        throwSaveError(filePath);
 }
 
 QScriptValue Scriptable::config()
 {
     m_skipArguments = -1;
 
-    QStringList nameValueInput;
-    nameValueInput.reserve( argumentCount() );
-
-    for (int i = 0; i < argumentCount(); ++i)
-        nameValueInput.append( arg(i) );
-
+    const auto nameValueInput = arguments();
     const auto result = m_proxy->config(nameValueInput);
     if ( result.type() == QVariant::String )
         return result.toString();
@@ -1467,7 +1537,7 @@ QScriptValue Scriptable::source()
 
     QByteArray script;
     {
-        QFile scriptFile( getFileName(scriptFilePath) );
+        QFile scriptFile( getAbsoluteFilePath(scriptFilePath) );
         if ( !scriptFile.open(QIODevice::ReadOnly) ) {
             throwError( QString("Failed to open \"%1\": %2")
                         .arg(scriptFilePath,
@@ -1507,8 +1577,8 @@ QScriptValue Scriptable::input()
         emit readInput();
         if (m_connected) {
             QEventLoop loop;
-            connect(this, SIGNAL(finished()), &loop, SLOT(quit()));
-            connect(this, SIGNAL(dataReceived()), &loop, SLOT(quit()));
+            connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
+            connect(this, &Scriptable::dataReceived, &loop, &QEventLoop::quit);
             loop.exec(QEventLoop::ExcludeUserInputEvents);
         }
     }
@@ -1659,12 +1729,6 @@ QScriptValue Scriptable::testSelected()
     m_skipArguments = 0;
     return m_proxy->testSelected();
 }
-
-void Scriptable::resetTestSession()
-{
-    m_skipArguments = 1;
-    m_proxy->resetTestSession( arg(0) );
-}
 #else // HAS_TESTS
 void Scriptable::keys()
 {
@@ -1676,11 +1740,6 @@ QScriptValue Scriptable::testSelected()
     m_skipArguments = 0;
     return QScriptValue();
 }
-
-void Scriptable::resetTestSession()
-{
-    m_skipArguments = 1;
-}
 #endif // HAS_TESTS
 
 void Scriptable::serverLog()
@@ -1816,13 +1875,7 @@ QScriptValue Scriptable::fromBase64()
 QScriptValue Scriptable::open()
 {
     m_skipArguments = -1;
-
-    for ( int i = 0; i < argumentCount(); ++i ) {
-        if ( !QDesktopServices::openUrl(QUrl(toString(argument(i), this))) )
-            return false;
-    }
-
-    return true;
+    return m_proxy->openUrls( arguments() );
 }
 
 QScriptValue Scriptable::execute()
@@ -1859,8 +1912,8 @@ QScriptValue Scriptable::execute()
     action.setCommand(args);
     action.setReadOutput(true);
 
-    connect( &action, SIGNAL(actionOutput(QByteArray)),
-             this, SLOT(onExecuteOutput(QByteArray)) );
+    connect( &action, &Action::actionOutput,
+             this, &Scriptable::onExecuteOutput );
 
     if ( !runAction(&action) || action.actionFailed() )
         return QScriptValue();
@@ -2029,14 +2082,12 @@ void Scriptable::sleep()
 
     if (m_connected) {
         QEventLoop loop;
-        connect(this, SIGNAL(finished()), &loop, SLOT(quit()));
+        connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
 
         QTimer t;
-#if QT_VERSION >= 0x050000
         t.setTimerType(Qt::PreciseTimer);
-#endif
         t.setInterval(msec);
-        connect(&t, SIGNAL(timeout()), &loop, SLOT(quit()));
+        connect(&t, &QTimer::timeout, &loop, &QEventLoop::quit);
         t.start();
 
         loop.exec();
@@ -2068,11 +2119,7 @@ void Scriptable::afterMilliseconds()
     public:
         TimedFunctionCall(int msec, const QScriptValue &fn, QObject *parent)
             : QObject(parent)
-#if QT_VERSION >= 0x050000
             , m_timerId( startTimer(msec, Qt::PreciseTimer) )
-#else
-            , m_timerId( startTimer(msec) )
-#endif
             , m_fn(fn)
         {
         }
@@ -2131,12 +2178,7 @@ QScriptValue Scriptable::screenshotSelect()
 
 QScriptValue Scriptable::screenNames()
 {
-#if QT_VERSION < 0x050000
-    throwError("Screen names are unsupported on version compiled with Qt 4");
-    return QScriptValue();
-#else
     return toScriptValue( m_proxy->screenNames(), this );
-#endif
 }
 
 QScriptValue Scriptable::queryKeyboardModifiers()
@@ -2198,7 +2240,9 @@ QScriptValue Scriptable::iconTagColor()
 void Scriptable::onClipboardChanged()
 {
     eval(R"(
-    if (runAutomaticCommands()) {
+    if (!hasData()) {
+        updateClipboardData();
+    } else if (runAutomaticCommands()) {
         synchronizeSelection();
         saveData();
         updateClipboardData();
@@ -2238,18 +2282,15 @@ void Scriptable::setTitle()
 
 void Scriptable::synchronizeSelection()
 {
+#ifdef HAS_MOUSE_SELECTIONS
     const bool syncToSelection = m_data.contains(mimeSyncToSelection);
     const bool syncToClipboard = m_data.contains(mimeSyncToClipboard);
     if (!syncToClipboard && !syncToSelection)
         return;
 
-    auto data = copyWithoutInternalData(m_data);
-
-    if (syncToSelection)
-        setClipboard(&data, ClipboardMode::Selection);
-
-    if (syncToClipboard)
-        setClipboard(&data, ClipboardMode::Clipboard);
+    const auto targetClipboardMode = syncToSelection ? ClipboardMode::Selection : ClipboardMode::Clipboard;
+    emit startSynchronizeSelectionTimer(targetClipboardMode, m_data);
+#endif
 }
 
 void Scriptable::saveData()
@@ -2262,6 +2303,24 @@ void Scriptable::saveData()
     }
 }
 
+QScriptValue Scriptable::hasData()
+{
+    for (auto it = m_data.constBegin(); it != m_data.constEnd(); ++it) {
+        const auto &format = it.key();
+        if ( isInternalDataFormat(format) )
+            continue;
+
+        auto bytes = it.value().toByteArray();
+        for (const auto &byte : bytes) {
+            const QChar c(byte);
+            if ( !c.isSpace() && !c.isNull() )
+                return true;
+        }
+    }
+
+    return false;
+}
+
 void Scriptable::showDataNotification()
 {
     m_proxy->showDataNotification(m_data);
@@ -2322,10 +2381,23 @@ void Scriptable::monitorClipboard()
 
     ClipboardMonitor monitor( monitorFormatsToSave() );
 
+#ifdef HAS_MOUSE_SELECTIONS
+    connect( QApplication::clipboard(), &QClipboard::changed,
+             this, [this]() {
+                m_data.remove(mimeSyncToSelection);
+                m_data.remove(mimeSyncToClipboard);
+                emit resetSynchronizeSelectionTimer();
+            });
+    SynchronizeSelectionTimer timer(m_proxy);
+    connect( this, &Scriptable::resetSynchronizeSelectionTimer, &timer, &SynchronizeSelectionTimer::reset );
+    connect( this, &Scriptable::startSynchronizeSelectionTimer, &timer, &SynchronizeSelectionTimer::start );
+#endif
+
     QEventLoop loop;
-    connect(this, SIGNAL(finished()), &loop, SLOT(quit()));
-    connect( &monitor, SIGNAL(runScriptRequest(QString,QVariantMap)),
-             this, SLOT(onMonitorRunScriptRequest(QString,QVariantMap)) );
+    connect(this, &Scriptable::finished, &loop, &QEventLoop::quit);
+    connect(this, &Scriptable::stop, &loop, &QEventLoop::quit);
+    connect( &monitor, &ClipboardMonitor::runScriptRequest,
+             this, &Scriptable::onMonitorRunScriptRequest );
     loop.exec();
 }
 
@@ -2339,11 +2411,6 @@ void Scriptable::provideSelection()
     provideClipboard(ClipboardMode::Selection);
 }
 
-void Scriptable::onDisconnected()
-{
-    abort();
-}
-
 void Scriptable::onExecuteOutput(const QByteArray &output)
 {
     m_executeStdoutData.append(output);
@@ -2361,20 +2428,26 @@ void Scriptable::onExecuteOutput(const QByteArray &output)
 
 void Scriptable::onMonitorRunScriptRequest(const QString &script, const QVariantMap &data)
 {
+    COPYQ_LOG("Monitor eval: " + script);
     m_data = data;
     m_proxy->setActionData(m_actionId, m_data);
     eval(script);
+    if ( engine()->hasUncaughtException() ) {
+        const auto exceptionText = processUncaughtException("ClipboardMonitor::" + script);
+        const auto response = createScriptErrorMessage(exceptionText).toUtf8();
+        sendMessageToClient(response, CommandException);
+    }
 }
 
 void Scriptable::onProvidedClipboardChanged()
 {
-    if ( !qApp->clipboard()->ownsClipboard() )
+    if ( !ownsClipboardData(ClipboardMode::Clipboard) )
         emit finished();
 }
 
 void Scriptable::onProvidedSelectionChanged()
 {
-    if ( !qApp->clipboard()->ownsSelection() )
+    if ( !ownsClipboardData(ClipboardMode::Selection) )
         emit finished();
 }
 
@@ -2382,9 +2455,11 @@ bool Scriptable::sourceScriptCommands()
 {
     const auto commands = m_proxy->scriptCommands();
     for (const auto &command : commands) {
+        engine()->pushContext();
         eval(command.cmd, command.name);
+        engine()->popContext();
         if ( engine()->hasUncaughtException() ) {
-            const auto exceptionText = processUncaughtException(command.cmd);
+            const auto exceptionText = processUncaughtException("ScriptCommand::" + command.cmd);
             const auto response = createScriptErrorMessage(exceptionText).toUtf8();
             sendMessageToClient(response, CommandException);
             return false;
@@ -2472,8 +2547,6 @@ void Scriptable::executeArguments(const QStringList &args)
     m_engine->collectGarbage();
 
     sendMessageToClient(response, exitCode);
-
-    COPYQ_LOG("DONE");
 }
 
 void Scriptable::setInput(const QByteArray &input)
@@ -2521,7 +2594,10 @@ void Scriptable::showExceptionMessage(const QString &message)
     const auto title = actionName.isEmpty()
         ? tr("Exception")
         : tr("Exception in %1").arg( quoteString(actionName) );
-    m_proxy->showMessage(title, message, QString(QChar(IconExclamationCircle)), 8000);
+
+    const auto id = qHash(title) ^ qHash(message);
+    const auto notificationId = QString::number(id);
+    m_proxy->showMessage(title, message, QString(QChar(IconExclamationCircle)), 8000, notificationId);
 }
 
 QVector<int> Scriptable::getRows() const
@@ -2608,6 +2684,11 @@ bool Scriptable::setClipboard(QVariantMap *data, ClipboardMode mode)
     return false;
 }
 
+void Scriptable::stopEventLoops()
+{
+    emit stop();
+}
+
 void Scriptable::changeItem(bool create)
 {
     int row;
@@ -2669,13 +2750,6 @@ QScriptValue Scriptable::screenshot(bool select)
     const auto screen = arg(1);
     const auto imageData = m_proxy->screenshot(format, screen, select);
 
-#if QT_VERSION < 0x050000
-    if ( !screen.isEmpty() ) {
-        throwError("Screen names are unsupported on version compiled with Qt 4");
-        return QScriptValue();
-    }
-#endif
-
     if ( imageData.isEmpty() ) {
         QString error = "Failed to grab screenshot";
         if ( !screen.isEmpty() ) {
@@ -2865,14 +2939,18 @@ void Scriptable::provideClipboard(ClipboardMode mode)
     clipboard->setData(mode, m_data);
 
     const auto slot = mode == ClipboardMode::Clipboard
-            ? SLOT(onProvidedClipboardChanged())
-            : SLOT(onProvidedSelectionChanged());
+            ? &Scriptable::onProvidedClipboardChanged
+            : &Scriptable::onProvidedSelectionChanged;
 
-    QTimer::singleShot(1000, this, slot);
+    QTimer t;
+    t.setInterval(8000);
+    connect(&t, &QTimer::timeout, this, slot);
+    t.start();
 
     QEventLoop loop;
-    connect( this, SIGNAL(finished()), &loop, SLOT(quit()) );
-    connect( clipboard.get(), SIGNAL(changed(ClipboardMode)), this, slot );
+    connect( this, &Scriptable::finished, &loop, &QEventLoop::quit );
+    connect( this, &Scriptable::stop, &loop, &QEventLoop::quit );
+    connect( clipboard.get(), &PlatformClipboard::changed, this, slot );
     loop.exec();
 }
 
@@ -2907,6 +2985,17 @@ void Scriptable::insert(int row, int argumentsBegin, int argumentsEnd)
         throwError(error);
 }
 
+QStringList Scriptable::arguments()
+{
+    QStringList args;
+    args.reserve( argumentCount() );
+
+    for (int i = 0; i < argumentCount(); ++i)
+        args.append( arg(i) );
+
+    return args;
+}
+
 QScriptValue NetworkReply::get(const QString &url, Scriptable *scriptable)
 {
     NetworkReply *reply = new NetworkReply(url, QByteArray(), scriptable);
