@@ -68,6 +68,7 @@ class Scriptable : public QObject, protected QScriptable
     Q_PROPERTY(QScriptValue mimeSyncToClipboard READ getMimeSyncToClipboard)
     Q_PROPERTY(QScriptValue mimeSyncToSelection READ getMimeSyncToSelection)
 
+    Q_PROPERTY(QScriptValue global READ getGlobal)
     Q_PROPERTY(QScriptValue plugins READ getPlugins)
 
 public:
@@ -107,11 +108,13 @@ public:
     QString getCurrentPath() const;
     void setCurrentPath(const QString &path);
 
-    QString getFileName(const QString &fileName) const;
+    QString getAbsoluteFilePath(const QString &fileName) const;
 
     QString arg(int i, const QString &defaultValue = QString());
 
     void throwError(const QString &errorMessage);
+    void throwSaveError(const QString &filePath);
+    void throwImportError(const QString &filePath);
 
     void sendMessageToClient(const QByteArray &message, int exitCode);
 
@@ -139,6 +142,7 @@ public:
     QScriptValue getMimeSyncToClipboard() const { return mimeSyncToClipboard; }
     QScriptValue getMimeSyncToSelection() const { return mimeSyncToSelection; }
 
+    QScriptValue getGlobal();
     QScriptValue getPlugins();
 
     ByteArrayClass *byteArrayClass() const { return m_baClass; }
@@ -151,6 +155,10 @@ public:
 
     void executeArguments(const QStringList &args);
 
+    bool setClipboard(QVariantMap *data, ClipboardMode mode);
+
+    void stopEventLoops();
+
 public slots:
     void setInput(const QByteArray &input);
 
@@ -216,8 +224,8 @@ public slots:
     void importTab();
     void importtab() { importTab(); }
 
-    QScriptValue importData();
-    QScriptValue exportData();
+    void importData();
+    void exportData();
 
     QScriptValue config();
     bool toggleConfig();
@@ -246,7 +254,6 @@ public slots:
 
     void keys();
     QScriptValue testSelected();
-    void resetTestSession();
     void serverLog();
 
     void setCurrentTab();
@@ -335,6 +342,7 @@ public slots:
     void setTitle();
     void synchronizeSelection();
     void saveData();
+    QScriptValue hasData();
     void showDataNotification();
     void hideDataNotification();
     void updateClipboardData();
@@ -349,29 +357,27 @@ public slots:
     void provideClipboard();
     void provideSelection();
 
-public slots:
-    void onDisconnected();
-
 signals:
     void sendMessage(const QByteArray &message, int messageCode);
     void dataReceived();
     void finished();
     void readInput();
+    void resetSynchronizeSelectionTimer();
+    void startSynchronizeSelectionTimer(ClipboardMode targetMode, const QVariantMap &data);
+    void stop();
 
-private slots:
+private:
     void onExecuteOutput(const QByteArray &output);
     void onMonitorRunScriptRequest(const QString &script, const QVariantMap &data);
     void onProvidedClipboardChanged();
     void onProvidedSelectionChanged();
 
-private:
     bool sourceScriptCommands();
     void callDisplayFunctions(QScriptValueList displayFunctions);
     QString processUncaughtException(const QString &cmd);
     void showExceptionMessage(const QString &message);
     QVector<int> getRows() const;
     QScriptValue copy(ClipboardMode mode);
-    bool setClipboard(QVariantMap *data, ClipboardMode mode);
     void changeItem(bool create);
     void nextToClipboard(int where);
     QScriptValue screenshot(bool select);
@@ -388,6 +394,8 @@ private:
     void insert(int argumentsEnd);
     void insert(int row, int argumentsBegin, int argumentsEnd);
 
+    QStringList arguments();
+
     ScriptableProxy *m_proxy;
     QScriptEngine *m_engine;
     ByteArrayClass *m_baClass;
