@@ -53,6 +53,7 @@
 #include <QComboBox>
 #include <QCursor>
 #include <QDateTimeEdit>
+#include <QDesktopServices>
 #include <QDesktopWidget>
 #include <QDialogButtonBox>
 #include <QFile>
@@ -61,6 +62,8 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QListWidget>
+#include <QMetaMethod>
+#include <QMetaType>
 #include <QMimeData>
 #include <QPainter>
 #include <QPaintEvent>
@@ -71,9 +74,13 @@
 #include <QShortcut>
 #include <QSpinBox>
 #include <QTextEdit>
+#include <QUrl>
 
 #include <type_traits>
 
+const quint32 serializedFunctionCallMagicNumber = 0x58746908;
+const quint32 serializedFunctionCallVersion = 1;
+
 #define BROWSER(call) \
     ClipboardBrowser *c = fetchBrowser(); \
     if (c) \
@@ -81,20 +88,21 @@
 
 #define STR(str) str
 
+#define INVOKE_(function, arguments) \
+    static auto f = FunctionCallSerializer(m_tabName, STR(#function)).withSlotArguments arguments; \
+    f.setArguments arguments; \
+    emit sendFunctionCall(f.serializeAndClear()) \
+
 #define INVOKE(function, arguments) \
     if (!m_wnd) { \
         using Result = decltype(function arguments); \
-        FunctionCallSerializer f(m_tabName, STR(#function), QVariant::fromValue(Result())); \
-        f.setArguments arguments; \
-        emit sendFunctionCall(f.serialize()); \
+        INVOKE_(function, arguments); \
         return m_returnValue.value<Result>(); \
     }
 
 #define INVOKE2(function, arguments) \
     if (!m_wnd) { \
-        FunctionCallSerializer f(m_tabName, STR(#function)); \
-        f.setArguments arguments; \
-        emit sendFunctionCall(f.serialize()); \
+        INVOKE_(function, arguments); \
         return; \
     }
 
@@ -225,30 +233,80 @@ QDataStream &operator>>(QDataStream &in, ScriptablePath &path)
     return in >> path.path;
 }
 
+QDataStream &operator<<(QDataStream &out, Qt::KeyboardModifiers value)
+{
+    return out << static_cast<int>(value);
+}
+
+QDataStream &operator>>(QDataStream &in, Qt::KeyboardModifiers &value)
+{
+    int valueInt;
+    in >> valueInt;
+    Q_ASSERT(in.status() == QDataStream::Ok);
+    value = static_cast<Qt::KeyboardModifiers>(valueInt);
+    return in;
+}
+
 namespace {
 
 const char propertyWidgetName[] = "CopyQ_widget_name";
 const char propertyWidgetProperty[] = "CopyQ_widget_property";
 
-const int noReturnType = -1;
-
 struct InputDialog {
     QDialog dialog;
     QString defaultChoice; /// Default text for list widgets.
+
+    explicit InputDialog(QWidget *parentWindow)
+        : dialog(parentWindow)
+    {
+    }
+};
+
+template<typename ...Ts>
+class SlotArguments;
+
+template<typename T, typename ...Ts>
+class SlotArguments<T, Ts...> {
+public:
+    static QByteArray arguments()
+    {
+        if ( std::is_same<QVariant, T>::value )
+            return "QVariant," + SlotArguments<Ts...>::arguments();
+
+        return QByteArray(QMetaType::typeName(qMetaTypeId<T>()))
+                + "," + SlotArguments<Ts...>::arguments();
+    }
+};
+
+template<>
+class SlotArguments<> {
+public:
+    static QByteArray arguments() { return QByteArray(); }
 };
 
 class FunctionCallSerializer {
 public:
-    explicit FunctionCallSerializer(const QString &tabName, const char *functionName, const QVariant &returnValue = noReturnType)
+    explicit FunctionCallSerializer(const QString &tabName, const char *functionName)
     {
-        m_args << tabName << QByteArray(functionName) << returnValue;
+        m_args << tabName << QByteArray(functionName);
     }
 
-    QByteArray serialize() const
+    template<typename ...Ts>
+    FunctionCallSerializer &withSlotArguments(Ts...)
+    {
+        QByteArray args = SlotArguments<Ts...>::arguments();
+        args.chop(1);
+        setSlotArgumentTypes(args);
+        return *this;
+    }
+
+    QByteArray serializeAndClear()
     {
         QByteArray bytes;
         QDataStream stream(&bytes, QIODevice::WriteOnly);
-        stream << m_args;
+        stream.setVersion(QDataStream::Qt_5_0);
+        stream << serializedFunctionCallMagicNumber << serializedFunctionCallVersion << m_args;
+        m_args.resize(2);
         return bytes;
     }
 
@@ -257,12 +315,22 @@ public:
     template<typename T, typename ...Ts>
     void setArguments(const T &head, Ts... args)
     {
-        m_args << std::is_same<QVariant, T>::value
-               << QVariant::fromValue(head);
+        m_args << QVariant::fromValue(head);
         setArguments(args...);
     }
 
 private:
+    void setSlotArgumentTypes(const QByteArray &args)
+    {
+        const auto slotName = m_args[1].toByteArray() + "(" + args + ")";
+        const int slotIndex = ScriptableProxy::staticMetaObject.indexOfSlot(slotName);
+        if (slotIndex == -1) {
+            log("Failed to find scriptable proxy slot: " + slotName, LogError);
+            Q_ASSERT(false);
+        }
+        m_args[1] = slotName;
+    }
+
     QVector<QVariant> m_args;
 };
 
@@ -566,6 +634,7 @@ ScriptableProxy::ScriptableProxy(MainWindow *mainWindow, QObject *parent)
     qRegisterMetaTypeStreamOperators<QVector<int>>("QVector<int>");
     qRegisterMetaTypeStreamOperators<QVector<Command>>("QVector<Command>");
     qRegisterMetaTypeStreamOperators<QVector<QVariantMap>>("QVector<QVariantMap>");
+    qRegisterMetaTypeStreamOperators<Qt::KeyboardModifiers>("Qt::KeyboardModifiers");
 }
 
 QByteArray ScriptableProxy::callFunction(const QByteArray &serializedFunctionCall)
@@ -573,38 +642,95 @@ QByteArray ScriptableProxy::callFunction(const QByteArray &serializedFunctionCal
     QVector<QVariant> functionCall;
     {
         QDataStream stream(serializedFunctionCall);
+        stream.setVersion(QDataStream::Qt_5_0);
+
+        quint32 magicNumber;
+        quint32 version;
+        stream >> magicNumber >> version;
+        if (stream.status() != QDataStream::Ok) {
+            log("Failed to read scriptable proxy slot call preamble", LogError);
+            Q_ASSERT(false);
+            return QByteArray();
+        }
+
+        if (magicNumber != serializedFunctionCallMagicNumber) {
+            log("Unexpected scriptable proxy slot call preamble magic number", LogError);
+            Q_ASSERT(false);
+            return QByteArray();
+        }
+
+        if (version != serializedFunctionCallVersion) {
+            log("Unexpected scriptable proxy slot call preamble version", LogError);
+            Q_ASSERT(false);
+            return QByteArray();
+        }
+
         stream >> functionCall;
+        if (stream.status() != QDataStream::Ok) {
+            log("Failed to read scriptable proxy slot call", LogError);
+            Q_ASSERT(false);
+            return QByteArray();
+        }
     }
 
     m_tabName = functionCall.value(0).toString();
-    const auto functionName = functionCall.value(1).toByteArray();
-    QVariant returnValue = functionCall.value(2);
-    const int argumentsStartIndex = 3;
-
-    QGenericArgument args[9];
-    for (int i = argumentsStartIndex, j = 0; i < functionCall.size(); i += 2, ++j) {
-        const auto isVariant = functionCall[i].toBool();
-        auto &value = functionCall[i + 1];
-        args[j] = isVariant
-                ? Q_ARG(QVariant, value)
-                : QGenericArgument( value.typeName(), static_cast<void*>(value.data()) );
-
+    const auto slotName = functionCall.value(1).toByteArray();
+    const auto slotIndex = metaObject()->indexOfSlot(slotName);
+    if (slotIndex == -1) {
+        log("Failed to find scriptable proxy slot: " + slotName, LogError);
+        Q_ASSERT(false);
+        return QByteArray();
     }
 
-    if (returnValue == noReturnType) {
-        const bool ok = QMetaObject::invokeMethod(
-                    this, functionName.constData(),
-                    args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
-        Q_ASSERT(ok);
-    } else {
-        const auto genericReturnValue = returnValue.isValid()
-                ? QGenericReturnArgument(returnValue.typeName(), static_cast<void*>(returnValue.data()) )
-                : Q_RETURN_ARG(QVariant, returnValue);
+    const int argumentsStartIndex = 2;
 
-        const bool ok = QMetaObject::invokeMethod(
-                    this, functionName.constData(), genericReturnValue,
-                    args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
-        Q_ASSERT(ok);
+    const auto metaMethod = metaObject()->method(slotIndex);
+    const auto typeId = metaMethod.returnType();
+
+    QGenericArgument args[9];
+    for (int i = argumentsStartIndex; i < functionCall.size(); ++i) {
+        auto &value = functionCall[i];
+        const int j = i - argumentsStartIndex;
+        const int argumentTypeId = metaMethod.parameterType(j);
+        if (argumentTypeId == QMetaType::QVariant) {
+            args[j] = Q_ARG(QVariant, value);
+        } else if ( value.userType() == argumentTypeId ) {
+            args[j] = QGenericArgument( value.typeName(), static_cast<void*>(value.data()) );
+        } else {
+            log( QString("Bad argument type (at index %1) for scriptable proxy slot: %2")
+                 .arg(j)
+                 .arg(metaMethod.methodSignature().constData()), LogError);
+            Q_ASSERT(false);
+            return QByteArray();
+        }
+    }
+
+    if (typeId == QMetaType::Void) {
+        const bool called = metaMethod.invoke(
+                this, args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
+
+        if (!called) {
+            log( QString("Bad scriptable proxy slot call: %1")
+                 .arg(metaMethod.methodSignature().constData()), LogError);
+            Q_ASSERT(false);
+        }
+
+        return QByteArray();
+    }
+
+    QVariant returnValue(typeId, nullptr);
+    const auto genericReturnValue = returnValue.isValid()
+            ? QGenericReturnArgument(returnValue.typeName(), static_cast<void*>(returnValue.data()) )
+            : Q_RETURN_ARG(QVariant, returnValue);
+
+    const bool called = metaMethod.invoke(
+                this, genericReturnValue,
+                args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8]);
+
+    if (!called) {
+        log("Bad scriptable proxy slot call (with return value)", LogError);
+        Q_ASSERT(false);
+        return QByteArray();
     }
 
     QByteArray bytes;
@@ -715,7 +841,7 @@ void ScriptableProxy::disableMonitoring(bool arg1)
 void ScriptableProxy::setClipboard(const QVariantMap &data, ClipboardMode mode)
 {
     INVOKE2(setClipboard, (data, mode));
-    m_wnd->setClipboard(data, mode);
+    m_wnd->setClipboardAndWait(data, mode);
 }
 
 QString ScriptableProxy::renameTab(const QString &arg1, const QString &arg2)
@@ -1249,12 +1375,6 @@ QString ScriptableProxy::testSelected()
 
     return browser->tabName() + " " + result.join(" ");
 }
-
-void ScriptableProxy::resetTestSession(const QString &clipboardTabName)
-{
-    INVOKE2(resetTestSession, (clipboardTabName));
-    m_wnd->resetTestSession(clipboardTabName);
-}
 #endif // HAS_TESTS
 
 void ScriptableProxy::serverLog(const QString &text)
@@ -1274,7 +1394,7 @@ NamedValueList ScriptableProxy::inputDialog(const NamedValueList &values)
 {
     INVOKE(inputDialog, (values));
 
-    InputDialog inputDialog;
+    InputDialog inputDialog(m_wnd);
     QDialog &dialog = inputDialog.dialog;
 
     QIcon icon;
@@ -1407,10 +1527,6 @@ QByteArray ScriptableProxy::screenshot(const QString &format, const QString &scr
 {
     INVOKE(screenshot, (format, screenName, select));
 
-#if QT_VERSION < 0x050000
-    auto pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
-    const auto geometry = QApplication::desktop()->geometry();
-#else
     QScreen *selectedScreen = nullptr;
     if ( screenName.isEmpty() ) {
         const auto mousePosition = QCursor::pos();
@@ -1435,7 +1551,6 @@ QByteArray ScriptableProxy::screenshot(const QString &format, const QString &scr
     auto pixmap = selectedScreen->grabWindow(0);
 
     const auto geometry = selectedScreen->geometry();
-#endif
 
     if (select) {
         ScreenshotRectWidget rectWidget(pixmap);
@@ -1449,13 +1564,9 @@ QByteArray ScriptableProxy::screenshot(const QString &format, const QString &scr
             QCoreApplication::processEvents();
         const auto rect = rectWidget.selectionRect;
         if ( rect.isValid() ) {
-#if QT_VERSION < 0x050000
-            pixmap = pixmap.copy(rect);
-#else
             const auto ratio = pixmap.devicePixelRatio();
             const QRect rect2( rect.topLeft() * ratio, rect.size() * ratio );
             pixmap = pixmap.copy(rect2);
-#endif
         }
     }
 
@@ -1472,9 +1583,6 @@ QByteArray ScriptableProxy::screenshot(const QString &format, const QString &scr
 
 QStringList ScriptableProxy::screenNames()
 {
-#if QT_VERSION < 0x050000
-    return QStringList();
-#else
     INVOKE(screenNames, ());
 
     QStringList result;
@@ -1485,7 +1593,6 @@ QStringList ScriptableProxy::screenNames()
         result.append(screen->name());
 
     return result;
-#endif
 }
 
 Qt::KeyboardModifiers ScriptableProxy::queryKeyboardModifiers()
@@ -1701,6 +1808,18 @@ QVector<Command> ScriptableProxy::scriptCommands()
     return m_wnd->scriptCommands();
 }
 
+bool ScriptableProxy::openUrls(const QStringList &urls)
+{
+    INVOKE(openUrls, (urls));
+
+    for (const auto &url : urls) {
+        if ( !QDesktopServices::openUrl(QUrl(url)) )
+            return false;
+    }
+
+    return true;
+}
+
 ClipboardBrowser *ScriptableProxy::fetchBrowser(const QString &tabName)
 {
     if (tabName.isEmpty()) {
