@@ -60,11 +60,6 @@ Q_DECLARE_METATYPE(Qt::KeyboardModifiers)
 Q_DECLARE_METATYPE(Command)
 Q_DECLARE_METATYPE(ClipboardMode)
 
-#if QT_VERSION < 0x050000
-Q_DECLARE_METATYPE(QVector<int>)
-Q_DECLARE_METATYPE(QVector<Command>)
-#endif
-
 QDataStream &operator<<(QDataStream &out, const NotificationButton &button);
 QDataStream &operator>>(QDataStream &in, NotificationButton &button);
 QDataStream &operator<<(QDataStream &out, const NamedValueList &list);
@@ -75,6 +70,8 @@ QDataStream &operator<<(QDataStream &out, ClipboardMode mode);
 QDataStream &operator>>(QDataStream &in, ClipboardMode &mode);
 QDataStream &operator<<(QDataStream &out, const ScriptablePath &path);
 QDataStream &operator>>(QDataStream &in, ScriptablePath &path);
+QDataStream &operator<<(QDataStream &out, Qt::KeyboardModifiers value);
+QDataStream &operator>>(QDataStream &in, Qt::KeyboardModifiers &value);
 
 class ScriptableProxy : public QObject
 {
@@ -188,7 +185,6 @@ public slots:
     void sendKeys(const QString &keys, int delay);
     bool keysSent();
     QString testSelected();
-    void resetTestSession(const QString &clipboardTabName);
 #endif // HAS_TESTS
 
     void serverLog(const QString &text);
@@ -241,6 +237,8 @@ public slots:
     QVector<Command> displayCommands();
     QVector<Command> scriptCommands();
 
+    bool openUrls(const QStringList &urls);
+
 signals:
     void sendFunctionCall(const QByteArray &bytes);
 
