@@ -20,9 +20,12 @@
 #include "tests.h"
 #include "test_utils.h"
 
+#include "common/appconfig.h"
 #include "common/client_server.h"
 #include "common/common.h"
+#include "common/config.h"
 #include "common/mimetypes.h"
+#include "common/settings.h"
 #include "common/shortcuts.h"
 #include "common/textdata.h"
 #include "common/version.h"
@@ -30,12 +33,15 @@
 #include "item/itemwidget.h"
 #include "item/serialize.h"
 #include "gui/configtabshortcuts.h"
+#include "gui/tabicons.h"
+#include "platform/platformnativeinterface.h"
 
 #include <QApplication>
 #include <QClipboard>
 #include <QDebug>
 #include <QDir>
 #include <QElapsedTimer>
+#include <QFile>
 #include <QFileInfo>
 #include <QMap>
 #include <QMimeData>
@@ -43,7 +49,7 @@
 #include <QRegExp>
 #include <QTemporaryFile>
 #include <QTest>
-#include <QTimerEvent>
+#include <QTimer>
 
 #include <memory>
 
@@ -53,38 +59,32 @@ const auto clipboardTabName = "CLIPBOARD";
 const auto defaultSessionColor = "#ff8800";
 const auto defaultTagColor = "#000000";
 
-/**
- * Run a process after a delay.
- *
- * This relies on other occational call to QCoreApplication::processEvents()
- * so the process can be started and others can run simultaneously.
- */
-class PostponedProcess : public QObject {
-public:
-    PostponedProcess(const TestInterfacePtr &test, const QStringList &args)
-        : m_test(test)
-        , m_args(args)
-    {
-        startTimer(2000);
-    }
-
-protected:
-    void timerEvent(QTimerEvent *event) override
-    {
-        killTimer( event->timerId() );
-        RUN(m_args, "");
-    }
-
-private:
-    TestInterfacePtr m_test;
-    QStringList m_args;
-};
+template <typename Fn1, typename Fn2>
+void runMultiple(Fn1 f1, int intervalMs, Fn2 f2)
+{
+    QTimer timer;
+    timer.setSingleShot(true);
+    timer.setInterval(intervalMs);
+    QObject::connect(&timer, &QTimer::timeout, f2);
+    timer.start();
+    f1();
+}
 
 bool testStderr(const QByteArray &stderrData, TestInterface::ReadStderrFlag flag = TestInterface::ReadErrors)
 {
-    const QRegExp re("^Warning:|^ERROR:|ASSERT", Qt::CaseInsensitive);
+    const QRegExp re("Warning:|ERROR:|ASSERT", Qt::CaseInsensitive);
+
+    QString output = QString::fromUtf8(stderrData);
+
+    // FIXME: These warnings should be fixed.
+    // Ignore known warnings.
+    output.remove("QtWarning: QXcbXSettings::QXcbXSettings(QXcbScreen*) Failed to get selection owner for XSETTINGS_S atom");
+    output.remove("QtWarning: QXcbConnection: XCB error:");
+    output.remove("QtWarning: libpng warning: iCCP: known incorrect sRGB profile");
+    output.remove("QtWarning: QMime::convertToMime: unhandled mimetype: text/plain");
+    output.remove("ERROR: QtCritical: QWindowsPipeWriter::write failed. (The pipe is being closed.)");
+    output.remove("ERROR: QtCritical: QWindowsPipeWriter: asynchronous write failed. (The pipe has been ended.)");
 
-    const QString output = QString::fromUtf8(stderrData);
     if ( output.indexOf(re) != -1 )
         return false;
 
@@ -199,29 +199,32 @@ public:
                 .toUtf8();
         }
 
-        // Wait for client/server communication is established.
-        SleepTimer t(15000);
-        while( !isServerRunning() && t.sleep() ) {}
-
-        if ( !isServerRunning() )
-            return "Unable to start server!" + readServerErrors(ReadAllStderr);
-
-        QByteArray errors = readServerErrors();
-        if (!errors.isEmpty())
-            return errors;
+        RETURN_ON_ERROR( waitForServerToStart(), "Failed to start server" );
+        RETURN_ON_ERROR( readServerErrors(), "Failed to read server errors" );
 
         return QByteArray();
     }
 
+    QByteArray waitForServerToStart()
+    {
+        SleepTimer t(15000);
+        do {
+            if ( isServerRunning() )
+                return QByteArray();
+        } while ( t.sleep() );
+
+        return "Unable to start server!" + readServerErrors(ReadAllStderr);
+    }
+
     QByteArray waitForAnyServerToQuit()
     {
         SleepTimer t(8000);
-        while ( isAnyServerRunning() && t.sleep() ) {}
+        do {
+            if ( !isAnyServerRunning() )
+                return QByteArray();
+        } while ( t.sleep() );
 
-        if ( isAnyServerRunning() )
-            return "Unable to stop server!" + readServerErrors(ReadAllStderr);
-
-        return QByteArray();
+        return "Unable to stop server!" + readServerErrors(ReadAllStderr);
     }
 
     QByteArray stopServer() override
@@ -256,21 +259,27 @@ public:
 
         p.closeWriteChannel();
 
-        if (stdoutData != nullptr)
+        if (stdoutData == nullptr)
+            p.closeReadChannel(QProcess::StandardOutput);
+        else
             stdoutData->clear();
 
-        if (stderrData != nullptr)
+        if (stderrData == nullptr)
+            p.closeReadChannel(QProcess::StandardError);
+        else
             stderrData->clear();
 
         SleepTimer t(waitClientRun);
         while ( p.state() == QProcess::Running ) {
-            const auto out = p.readAllStandardOutput();
-            const auto err = p.readAllStandardError();
-
-            if (stdoutData != nullptr)
+            if ( stdoutData != nullptr ) {
+                const auto out = p.readAllStandardOutput();
                 stdoutData->append(out);
-            if (stderrData != nullptr)
+            }
+
+            if (stderrData != nullptr) {
+                const auto err = p.readAllStandardError();
                 stderrData->append(err);
+            }
 
             if ( !t.sleep() ) {
                 qWarning() << "client process timed out";
@@ -370,13 +379,7 @@ public:
         QApplication::clipboard()->setMimeData(mimeData);
 
         waitUntilClipboardSet(bytes, mime);
-        QByteArray error = testClipboard(bytes, mime);
-        if ( !error.isEmpty() )
-            return "Failed to set clipboard! " + error;
-
-        error = testClipboard(bytes, mime);
-        if ( !error.isEmpty() )
-            return "Clipboard was unexpectedly changed! " + error;
+        RETURN_ON_ERROR( testClipboard(bytes, mime), "Failed to set clipboard" );
 
         return "";
     }
@@ -420,9 +423,7 @@ public:
         if ( !testStderr(stderrActual) || exitCode != 0 )
             return printClienAndServerStderr(stderrActual, exitCode);
 
-        const QByteArray errors = readServerErrors();
-        if ( !errors.isEmpty() )
-            return errors;
+        RETURN_ON_ERROR( readServerErrors(), "Failed to read server errors" );
 
         return "";
     }
@@ -446,23 +447,51 @@ public:
 
     QByteArray init() override
     {
-        auto errors = cleanup();
-        if ( !errors.isEmpty() )
-            return errors;
+        RETURN_ON_ERROR( cleanup(), "Failed to cleanup" );
 
-        if ( !isServerRunning() ) {
-            errors = startServer();
-            if ( !errors.isEmpty() )
-                return "Failed to start server:\n" + errors;
+        if ( isServerRunning() )
+            RETURN_ON_ERROR( stopServer(), "Failed to stop server" );
+
+        // Remove all configuration.
+        const auto settingsPath = settingsDirectoryPath();
+        QDir settingsDir(settingsPath);
+        if ( settingsDir.exists() && !settingsDir.removeRecursively() )
+            return "Failed to remove settings directory " + settingsPath.toUtf8();
+
+        // Update settings for tests.
+        {
+            Settings settings;
+            settings.clear();
+
+            settings.beginGroup("Options");
+            settings.setValue( Config::clipboard_tab::name(), clipboardTabName );
+            settings.setValue( Config::close_on_unfocus::name(), false );
+            settings.endGroup();
+
+            if ( !m_settings.isEmpty() ) {
+                const bool pluginsTest = m_testId != "CORE";
+
+                if (pluginsTest) {
+                    settings.beginGroup("Plugins");
+                    settings.beginGroup(m_testId);
+                }
+
+                for (auto it = m_settings.constBegin(); it != m_settings.constEnd(); ++it)
+                    settings.setValue( it.key(), it.value() );
+
+                if (pluginsTest) {
+                    settings.endGroup();
+                    settings.endGroup();
+                }
+            }
         }
 
-        errors = setClipboard(QByteArray(), mimeText);
-        if ( !errors.isEmpty() )
-            return "Failed to reset clipboard:\n" + errors;
+        verifyConfiguration();
 
-        errors = runClient(Args("resetTestSession") << clipboardTabName, "");
-        if ( !errors.isEmpty() )
-            return errors;
+        // Clear clipboard.
+        RETURN_ON_ERROR( setClipboard(QByteArray(), mimeText), "Failed to reset clipboard" );
+
+        RETURN_ON_ERROR( startServer(), "Failed to start server" );
 
         // Always show main window first so that the results are consistent with desktop environments
         // where user cannot hide main window (tiling window managers without tray).
@@ -481,14 +510,22 @@ public:
 
     void setupTest(const QString &id, const QVariant &settings)
     {
+        m_testId = id;
+        m_settings = settings.toMap();
         m_env.insert("COPYQ_TEST_ID", id);
-        QByteArray data;
-        QDataStream out(&data, QIODevice::WriteOnly);
-        out << settings;
-        m_env.insert("COPYQ_TEST_SETTINGS", data.toBase64());
     }
 
 private:
+    void verifyConfiguration()
+    {
+        AppConfig appConfig;
+        QCOMPARE( appConfig.option<Config::close_on_unfocus>(), false );
+        QCOMPARE( appConfig.option<Config::clipboard_tab>(), QString(clipboardTabName) );
+        QCOMPARE( appConfig.option<Config::maxitems>(), Config::maxitems::defaultValue() );
+        QCOMPARE( savedTabs(), QStringList(clipboardTabName) );
+        QCOMPARE( tabs(), QStringList() );
+    }
+
     bool isAnyServerRunning()
     {
         return run(Args("eval")) == 0;
@@ -517,6 +554,7 @@ private:
 
     std::unique_ptr<QProcess> m_server;
     QProcessEnvironment m_env;
+    QString m_testId;
     QVariantMap m_settings;
 };
 
@@ -601,8 +639,6 @@ void Tests::commandVersion()
     QVERIFY( version.contains(QRegExp("\\bCopyQ\\b.*" + QRegExp::escape(COPYQ_VERSION))) );
     // Version contains Qt version.
     QVERIFY( version.contains(QRegExp("\\bQt\\s+\\d")) );
-    // Version contains Qxt version.
-    QVERIFY( version.contains(QRegExp("\\bLibQxt\\s+\\d")) );
 }
 
 void Tests::badCommand()
@@ -864,35 +900,41 @@ void Tests::commandToggleConfig()
 
 void Tests::commandDialog()
 {
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << ":TEST" << "ENTER");
-        RUN("dialog" << "text", "TEST\n");
-    }
+    runMultiple(
+        [&]() { RUN("dialog" << "text", "TEST\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << ":TEST" << "ENTER", ""); }
+    );
 
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << "ESCAPE");
-        RUN("eval" << "dialog('text') === undefined", "true\n");
-    }
+    runMultiple(
+        [&]() { RUN("eval" << "dialog('text') === undefined", "true\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << "ESCAPE", ""); }
+    );
 
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << "ENTER");
-        RUN("eval" << "dialog('.defaultChoice', 2, 'list', [1, 2, 3])", "2\n");
-    }
+    runMultiple(
+        [&]() { RUN("eval" << "dialog('.defaultChoice', 2, 'list', [1, 2, 3])", "2\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << "ENTER", ""); }
+    );
 
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << "ENTER");
-        RUN("eval" << "dialog('.defaultChoice', '', 'list', [1, 2, 3])", "\n");
-    }
+    runMultiple(
+        [&]() { RUN("eval" << "dialog('.defaultChoice', '', 'list', [1, 2, 3])", "\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << "ENTER", ""); }
+    );
 
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << "ENTER");
-        RUN("eval" << "dialog('list', [0, 1, 2])", "0\n");
-    }
+    runMultiple(
+        [&]() { RUN("eval" << "dialog('list', [0, 1, 2])", "0\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << "ENTER", ""); }
+    );
 
-    {
-        PostponedProcess dialogThread(m_test, Args() << "keys" << "ENTER");
-        RUN("eval" << "dialog('boolean', true) === true", "true\n");
-    }
+    runMultiple(
+        [&]() { RUN("eval" << "dialog('boolean', true) === true", "true\n"); },
+        waitMsShow,
+        [&]() { RUN(Args() << "keys" << "ENTER", ""); }
+    );
 }
 
 void Tests::commandsPackUnpack()
@@ -1113,6 +1155,8 @@ void Tests::commandHasClipboardFormat()
 
 void Tests::commandEdit()
 {
+    SKIP_ON_ENV("COPYQ_TESTS_SKIP_COMMAND_EDIT");
+
     RUN("config" << "editor" << "", "\n");
 
     // Edit new item.
@@ -1176,14 +1220,14 @@ void Tests::commandsExportImport()
     tmp.close();
     const auto fileName = tmp.fileName();
 
-    RUN("exportData" << fileName, "true\n");
+    RUN("exportData" << fileName, "");
 
     RUN("config" << "maxitems" << "1", "1\n");
     RUN("config" << "editor" << "EDITOR2 %1", "EDITOR2 %1\n");
     RUN("removetab" << tab1, "");
     RUN("tab" << tab2 << "add" << "1", "");
 
-    RUN("importData" << fileName, "true\n");
+    RUN("importData" << fileName, "");
 
     RUN("config" << "maxitems", "3\n");
     RUN("config" << "editor", "EDITOR1 %1\n");
@@ -1433,11 +1477,6 @@ void Tests::moveItems()
     RUN(args << "read" << "0" << "1" << "2", "B A C");
     RUN(args << "testSelected", tab + " 1 1\n");
 
-    // WORKAROUND: In Qt 4.8.7, selection is broken after moving items.
-#if QT_VERSION < 0x050000
-    RUN(args << "keys" << "UP" << "DOWN", "");
-#endif
-
     // move items to top
     RUN(args << "keys" << "SHIFT+DOWN" << "CTRL+HOME", "");
     RUN(args << "read" << "0" << "1" << "2", "A C B");
@@ -1646,7 +1685,7 @@ void Tests::tabRemove()
 void Tests::tabIcon()
 {
     const QString tab = testTab(1);
-    const QString icon = ":/images/icon.svg";
+    const QString icon = ":/images/icon";
 
     RUN("tab" << tab << "add" << "", "");
     RUN("tabIcon" << tab, "\n");
@@ -1874,7 +1913,7 @@ void Tests::externalEditor()
 
     // Check if clipboard changed.
     WAIT_ON_OUTPUT("read" << "0", data1 + data2);
-    RUN("clipboard", data1 + data2);
+    WAIT_FOR_CLIPBOARD(data1 + data2);
 
     // Edit existing item.
     const QString text =
@@ -1990,17 +2029,21 @@ void Tests::pasteFromMainWindow()
 
     RUN("add" << "TEST", "");
     RUN("hide", "");
-    const auto pasteFromMainWindowScript = QString(
-                R"(
-                show()
-                sleep(%1)
-                keys('ENTER')
-                sleep(%1)
-                keys('ENTER')
-                )"
-                ).arg(waitMsShow);
-    PostponedProcess dialogThread(m_test, Args("eval") << pasteFromMainWindowScript);
-    RUN("dialog" << "text", "TEST\n");
+    runMultiple(
+        [&]() { RUN("dialog" << "text", "TEST\n"); },
+        waitMsShow,
+        [&]() {
+            RUN("show", "");
+            waitFor(waitMsShow);
+            RUN("keys" << "ENTER", "");
+            waitFor(waitMsShow);
+
+            WAIT_FOR_CLIPBOARD("TEST");
+            waitFor(waitMsPasteClipboard);
+
+            RUN("keys" << "ENTER", "");
+        }
+    );
 }
 
 void Tests::tray()
@@ -2058,6 +2101,7 @@ void Tests::trayPaste()
     waitFor(waitMsShow);
     RUN("keys" << "ENTER", "");
     WAIT_FOR_CLIPBOARD("TEST");
+    waitFor(waitMsPasteClipboard);
 
     RUN("keys" << "F2", "");
     RUN("tab" << tab1 << "read" << "0", "NEW TEST");
@@ -2105,8 +2149,12 @@ void Tests::configTrayTab()
 
 void Tests::configMove()
 {
+    SKIP_ON_ENV("COPYQ_TESTS_SKIP_CONFIG_MOVE");
+
     RUN("add" << "B" << "A", "");
 
+    RUN("config" << "tray_item_paste" << "false", "false\n");
+
     RUN("config" << "move" << "true", "true\n");
 
     waitFor(waitTrayUpdate);
@@ -2149,6 +2197,19 @@ void Tests::configTrayTabIsCurrent()
     WAIT_FOR_CLIPBOARD("B");
 }
 
+void Tests::configAutostart()
+{
+    if ( !createPlatformNativeInterface()->canAutostart() ) {
+        SKIP("Autostart is unsupported on this platform");
+        return;
+    }
+
+    RUN("config" << "autostart" << "true", "true\n");
+    RUN("config" << "autostart", "true\n");
+    RUN("config" << "autostart" << "false", "false\n");
+    RUN("config" << "autostart", "false\n");
+}
+
 void Tests::shortcutCommand()
 {
     RUN("setCommands([{name: 'test', inMenu: true, shortcuts: ['Ctrl+F1'], cmd: 'copyq add OK'}])", "");
@@ -2453,7 +2514,7 @@ void Tests::scriptCommandAddFunction()
     const auto script = R"(
         setCommands([{
             isScript: true,
-            cmd: 'test = function() { return "TEST"; }'
+            cmd: 'global.test = function() { return "TEST"; }'
         }])
         )";
     RUN(script, "");
@@ -2474,23 +2535,39 @@ void Tests::scriptCommandOverrideFunction()
 
 void Tests::displayCommand()
 {
+    const auto testMime = COPYQ_MIME_PREFIX "test";
     const auto script = QString(R"(
         setCommands([{
             display: true,
             input: '%1',
             cmd: 'copyq:'
-               + 'var data = unpack(input());'
-               + 'text = str(data[mimeText]);'
-               + 'if (text !== "a" && text != "b") abort();'
-               + 'add("CALLED in " + str(data[mimeCurrentTab]));'
+               + 'text = str(data(mimeText));'
+               + 'currentTab = str(data(mimeCurrentTab));'
+               + 'add(currentTab + "/" + text);'
         }])
-        )").arg(mimeItems);
-    RUN("add('b', 'a'); " + QString(script), "");
+        )").arg(testMime);
+
+    RUN(script, "");
+
+    RUN("write" << "0" << testMime << "" << mimeText << "a", "");
     WAIT_ON_OUTPUT(
-                "read(0,1,2,3,4)",
-                QString("CALLED in %1\nCALLED in %1\na\nb\n")
+                "read(0,1,2)",
+                QString("%1/a\na\n")
                 .arg(clipboardTabName)
                 .toUtf8() );
+
+    RUN("write" << "0" << testMime << "" << mimeText << "b", "");
+    WAIT_ON_OUTPUT(
+                "read(0,1,2,3,4)",
+                QString("%1/b\nb\n%1/a\na\n")
+                .arg(clipboardTabName)
+                .toUtf8() );
+}
+
+void Tests::queryKeyboardModifiersCommand()
+{
+    RUN("queryKeyboardModifiers()", "");
+    // TODO: Is there a way to press modifiers?
 }
 
 int Tests::run(const QStringList &arguments, QByteArray *stdoutData, QByteArray *stderrData, const QByteArray &in)
@@ -2526,6 +2603,13 @@ int runTests(int argc, char *argv[])
     }
 
     QApplication app(argc, argv);
+
+    const QString session = "copyq.test";
+    QCoreApplication::setOrganizationName(session);
+    QCoreApplication::setApplicationName(session);
+    Settings::canModifySettings = true;
+    createPlatformNativeInterface()->loadSettings();
+
     int exitCode = 0;
     std::shared_ptr<TestInterfaceImpl> test(new TestInterfaceImpl);
     Tests tc(test);
