@@ -177,6 +177,8 @@ private slots:
     void configMove();
     void configTrayTabIsCurrent();
 
+    void configAutostart();
+
     void shortcutCommand();
     void shortcutCommandOverrideEnter();
     void shortcutCommandMatchInput();
@@ -202,6 +204,8 @@ private slots:
     void scriptCommandOverrideFunction();
     void displayCommand();
 
+    void queryKeyboardModifiersCommand();
+
 private:
     void clearServerErrors();
     int run(const QStringList &arguments, QByteArray *stdoutData = nullptr,
